@@ -13,16 +13,25 @@
 ##############################################################################
 """Session implementation using cookies
 
-$Id: http.py 96645 2009-02-17 18:00:24Z nadako $
+$Id: http.py 98162 2009-03-16 21:12:35Z nadako $
 """
 import hmac
 import logging
 import random
 import re
-import sha
 import string
 import time
 from cStringIO import StringIO
+try:
+    from hashlib import sha1
+except ImportError:
+    # Python 2.4
+    from sha import new as sha1
+try:
+    from email.utils import formatdate
+except ImportError:
+    # Python 2.4
+    from email.Utils import formatdate
 
 import zope.location
 from persistent import Persistent
@@ -30,13 +39,10 @@ from zope import schema, component
 from zope.interface import implements
 from zope.publisher.interfaces.http import IHTTPRequest
 from zope.publisher.interfaces.http import IHTTPApplicationRequest
-from zope.annotation.interfaces import IAttributeAnnotatable
 from zope.i18nmessageid import ZopeMessageFactory as _
 from zope.session.interfaces import IClientIdManager
 from zope.schema.fieldproperty import FieldProperty
 
-from zope.app.http.httpdate import build_http_date
-
 __docformat__ = 'restructuredtext'
 
 cookieSafeTrans = string.maketrans("+/", "-.")
@@ -124,7 +130,7 @@ class ICookieClientIdManager(IClientIdManager):
 class CookieClientIdManager(zope.location.Location, Persistent):
     """Session utility implemented using cookies."""
 
-    implements(IClientIdManager, ICookieClientIdManager, IAttributeAnnotatable)
+    implements(IClientIdManager, ICookieClientIdManager)
 
     thirdparty = FieldProperty(ICookieClientIdManager['thirdparty'])
     cookieLifetime = FieldProperty(ICookieClientIdManager['cookieLifetime'])
@@ -251,11 +257,11 @@ class CookieClientIdManager(zope.location.Location, Persistent):
 
         """
         data = "%.20f%.20f%.20f" % (random.random(), time.time(), time.clock())
-        digest = sha.sha(data).digest()
+        digest = sha1(data).digest()
         s = digestEncode(digest)
         # we store a HMAC of the random value together with it, which makes
         # our session ids unforgeable.
-        mac = hmac.new(s, self.secret, digestmod=sha).digest()
+        mac = hmac.new(s, self.secret, digestmod=sha1).digest()
         return s + digestEncode(mac)
 
     def getRequestId(self, request):
@@ -339,7 +345,7 @@ class CookieClientIdManager(zope.location.Location, Persistent):
             # call encode() on value s a workaround a bug where the hmac
             # module only accepts str() types in Python 2.6
             if (digestEncode(hmac.new(
-                    s.encode(), self.secret, digestmod=sha
+                    s.encode(), self.secret, digestmod=sha1
                 ).digest()) != mac):
                 return None
             else:
@@ -449,7 +455,8 @@ class CookieClientIdManager(zope.location.Location, Persistent):
         options = {}
         if self.cookieLifetime is not None:
             if self.cookieLifetime:
-                expires = build_http_date(time.time() + self.cookieLifetime)
+                expires = formatdate(time.time() + self.cookieLifetime,
+                                     localtime=False, usegmt=True)
             else:
                 expires = 'Tue, 19 Jan 2038 00:00:00 GMT'
 
